# -*- coding: UTF-8 -*-

#   icd_cyl_cam.py     2020.11.13  kita
#    円筒板カム作成  script
#       新規図面で実施

#   必要なファイル
#        universal_cam.py (editor.ini icd_cam.py) 

import numpy as np
import clr                  # .NETをimport　
clr.AddReference('sxnet')   # 
from sxnet import *     # 全てのクラスをimport
SxSys.init(3999)

import os
import subprocess
import tkinter as tk
import tkinter.ttk as ttk       # ComboBoxを使うのに必要
import universal_cam as uvc

# Editorの設定
fn = 'editor.ini'


if os.path.exists(fn):      # fnが存在するなら
    fp = open(fn, 'r')
    for line in fp.readlines():
        if line.find('#') > -1:  # #が含まれてるなら
            continue
        line = line.rstrip()    # 行末の改行を削除
        tmp = line.split(',')
        if tmp[0] == 'editor':
            editor = tmp[1]
else:
    editor = 'notepad'

#print(editor)


#----------------
# 諸関数
#----------------

def push_help():
    def x():
        cmd = editor +' icd_sld_cam.txt'
        result = subprocess.call(cmd,shell=True)
    return x

def get_cam():  # Cam No 、作図条件取得
    cam_nm = v_cam.get()
    if cam_nm.find('変形正弦') > -1:
        cam_no = 26
    elif  cam_nm.find('変形台形') > -1:
        cam_no = 25
    elif  cam_nm.find('変形等速度') > -1:
        cam_no = 27
    elif  cam_nm.find('Cycroidal') > -1:
        cam_no = 22
    cam = uvc.Universal_cam(cam_no)
    r_in = float(entry[0].get())
    r_out = float(entry[1].get())
    h_r = float(entry[2].get())
    th_h = float(entry[3].get())
    d_th = float(entry[4].get())
    r_rol = float(entry[5].get())/2.0
    h0 = float(entry[6].get())

    return (cam, r_in, r_out, h_r, th_h, d_th, r_rol,  h0)

def make_pln_yth(yth_d, gen=[0.0,0.0,0.0]): # yth=0[deg] -> x軸
    yth_r = yth_d*np.pi/180     # [deg] ->[rad]
    pos = SxPos()
    pos.x = gen[0]
    pos.y = gen[1]
    pos.z = gen[2]
    #pos = SxPos(gen[0], gen[1], gen[2]) # pos.x = 0にしかならない
    #print(pos.x, gen[0])
    xvec = SxVec(np.cos(yth_r), 0.0, -np.sin(yth_r))
    zvec = SxVec(np.sin(yth_r), 0.0, np.cos(yth_r))
    plan = SxAsmPlane.create(pos, zvec, xvec)

    return plan

def sakuzu_pp(*pp):  # p = []
    nn = len(pp)
    seg_lst = []
    #print(nn)
    for i in range(nn):
        pos0 = SxPos(pp[i][0],pp[i][1],pp[i][2])
        if i == nn-1:
            pos1 = SxPos(pp[0][0],pp[0][1],pp[0][2])
        else:
            pos1 = SxPos(pp[i+1][0],pp[i+1][1],pp[i+1][2])
        #print(pos0.x,pp[i][0])    
        seg = SxEntSeg.createLine2D(pos0,pos1)
        seg_lst.append(seg)

    return seg_lst


def push_exe():
    def x():
        (cam, r_in, r_out, h_r, th_h, d_th, r_rol, h0) = get_cam()
        r_c = (r_in+r_out)/2    # 板厚中心半径
        th_h_rad = th_h*np.pi/180.0     # 割付角[rad]
        #print(th_h_rad)
        nn = int(th_h/d_th)
        z = 0.0
        p00 = [r_in,0.0,z]; p01 = [r_out,0.0, z] 
        pos0 = SxPos(p00[0], p00[1], p00[2])
        rpos_list = []      # 複数閉領域の基準点座標配列
        seg_lsts = []       # Segment List
        plan = []
        org_list = None #ビューの原点に配置
        
        #debug = 1  # debug mode
        debug = 0   # 作図実行 mode

        for i in range(round(nn+1)):
            #print(i)
            rpos_list.append(pos0)
            t = i/nn
            (s, v, a) = cam.sva(t)
            # 組立平面設置個所の補正
            #th_pr = np.arctan2(h_r*v, r_out*th_h_rad) # 圧力角[rad]    外径
            th_pr = np.arctan2(h_r*v, r_c*th_h_rad) # 圧力角[rad]  板厚中心
            th_k = (r_rol*np.sin(th_pr)/r_c) * 180/np.pi #補正角度[deg] 弧長さ
            #th_k = np.arcsin(r_rol*np.sin(th_pr)/r_c) * 180/np.pi #補正角度[deg]　x軸投影長さ
            s_r = h0 + s*h_r + r_rol*(1.0-np.cos(th_pr))
            th = i*d_th + th_k

            if debug == 1:  # debug mode
                print(i, s_r, th_pr*180/np.pi)
            else :    
                # @Plan[i]
                plan.append(make_pln_yth(th))
                pp = [p00,p01,[r_out,s_r,z],[r_in,s_r,z]]
                seg_lsts.append(sakuzu_pp(*pp))
        
        if debug != 1:  # 作図モード
            rseg = SxEntSeg.createLoft3D(seg_lsts,rpos_list,org_list,True,False,True,False)   # Loft作成
                                               #接線接続断面 開始, 中間, 終了, 実像部品を引き継がない     
            for i in range(round(nn+1)):
                plan[i].delete()  #組立平面削除
    return x
        
#************************************
# ボタン等の部品配置
#************************************
nl = 9
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title('円筒カム作図 Ver.0.8')   #タイトル
root.geometry('200x%d+700+0' % (nl*30))


btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'



#-------------------------------------------
#   Wigit配置
#-------------------------------------------
btn_help = tk.Button(root,text='ヘルプ', command = push_help(), bg = btn_bcol, fg = btn_tcol)  # Buttonの作成
btn_exe = tk.Button(root,text='実行', command = push_exe(), bg = btn_bcol, fg = btn_tcol)  # Buttonの作成

btn_help.grid(column=0,row=nl,padx=5, pady=5)
btn_exe.grid(column=1,row=nl,padx=5, pady=5)

#---------------------------------------------------
ss = 'カ ム 曲 線' ; nx = 0; ny = 1  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
#label.place(x=x00+nx*dx,y=y00+ny*dy)
label.grid(row=0, column=0, padx=5, pady=5)
        
v_cam = tk.StringVar()
cb = ttk.Combobox(root, width=10,textvariable = v_cam) #, style="BW.TLabel")
cb['values']=('変形正弦','変形台形','変形等速度','Cycroidal')
cb.set('変形正弦')
#cb.place(x=x00+(nx+1.3*dx),y=y00+ny*dy)
cb.grid(row=0, column=1)    # 行、列


label = []
label1 = []
entry = []
label_txt = ['r_in_ 内径', 'r_out_外径', 'CamStroke ', ' 割 付 角  ', '微小割付角','ローラー径',' 基礎高さ  '] 
label1_txt = ['[mm]', '[mm]', '[mm]', '[deg]', '[deg]','[mm]','[mm]'] 
entry_txt = ['349','361','175','109','1','67','43.5']

for i in range(len(label_txt)):
    label.append(tk.Button(root, text=label_txt[i], bg=label_bcol,fg=label_tcol))
    label1.append(tk.Button(root, text=label1_txt[i], bg=label_bcol,fg=label_tcol))
    entry.append(tk.Entry(root, width=6,justify="right",bg=e_bcol)) 
    entry[i].insert(0,entry_txt[i])
    #entry[i].insert(tk.END,etry_txt[i])
    label[i].grid(row=i+1, column=0, padx=1, pady=1)
    label1[i].grid(row=i+1, column=3, padx=1, pady=1)
    entry[i].grid(row=i+1, column=1, padx=1, pady=1)



root.mainloop()        
