#!/usr/bin/env python3
# -*- coding: UTF-8 -*- 
#   universal_cam.py  2020.7.25   kita
#   

import numpy as np
try:
    import matplotlib.pyplot as plt
except:
    print('matplotlib がインストールされていません。グラフの作図機能は使えません')

"""
__init__(self,cam_no,z,*tt):
cam_no      22: Cycroidal
            25:標準変形台形         26:標準変形正弦        27:標準変形等速度
            45:片停留変形台形(m=1)  48:片停留変形正弦
            51:無停留変形台形       52:無停留変形等速度
            1:早送り(v7!=0)         2:切削送り(等速->停止)
            0:t0～t7を指定する	    3:全ての変数を指定
            1: z   早送り曲線のパラメータ　t3 = 1/(1+z)
            0:*tt   t0-t7 をListで指定する場合に使用する
  
sva(self,t):  時間t(0-1)の時の(s, v, a)を返す。

sva_nn(self,nn,mode):   t=0-1を nn 分割した(s, v, a)を返す。mode=1 でPlot out


"""

class Universal_cam:
    # universal_camの初期化  インスタンス（実体）作成時に実行される
    def __init__(self, cam_no, z=0.9, ratio=1.0, *tt):
        s0 = 0; s7 = 1; v0 = 0; v7 = 0; pi = np.pi #; ratio = 1
        if cam_no == 22:    # Cycroidal
            tt = [0.0,1/4,1/4,1/2,1/2,3/4,3/4,1.0]; n_a0 = 2; ttl = 'Cycroidal'
        elif cam_no == 25:    #　標準変形台形
            tt = [0.0,1/8,3/8,1/2,1/2,5/8,7/8,1.0]; n_a0 = 2; ttl = 'Std_Modified_Trapezoid' #'標準変形台形'
        elif cam_no == 26:    #　標準変形正弦
            tt = [0.0,1/8,1/8,1/2,1/2,7/8,7/8,1.0]; n_a0 = 2; ttl = 'Std_Modified_Sine'    #'標準変形正弦'
        elif cam_no == 27:    #　標準変形等速度
            tt = [0.0,1/16,1/16,1/4,3/4,15/16,15/16,1.0]; n_a0 = 2; ttl = 'Std_Mod_Const_Velocity'   #標準変形等速度'
        elif cam_no == 45:    #　片停留変形台形
            tt = [0.0,1/8,3/8,1/2,1/2,5/8,1.0,1.0]; n_a0 = 1; ttl = 'One_Dwell_Mod_Trapezoid'  #片停留変形台形'
        elif cam_no == 48:    #　片停留変形正弦
            tt = [0.0,1/8,1/8,4/8,4/8,1.0,1.0,1.0]; n_a0 = 1; ttl = 'One_Dwell_Mod_Sine'  #片停留変形正弦'
        elif cam_no == 51:    #　無停留変形台形
            t = [0.0,0,2/8,4/8,4/8,6/8,1.0,1.0]; n_a0 = 0; ttl = 'No_Dwell_Mod_Trapezoid'  #無停留変形台形'
        elif cam_no == 52:    #　無停留変形等速度
            tt = [0.0,0,2/8,4/8,4/8,6/8,1.0,1.0]; n_a0 = 0; ttl = 'No_Dwell_Mod_Cst_Velosity'  #無停留変形等速度'
        elif cam_no == 1:     #  早送り(v7!=0)
            t0 = 0; t3 = 1/(1+z); t1 = t3/4; t2 = t1; t7 = 1; t5 = t7-(t7-t3)/4; t4 = t3; t6 = t5; n_a0 = 2;
            tt = [t0,t1,t2,t3,t4,t5,t6,t7]; ttl = 'hayaokuri(z=%1.1f)' % z     
        elif cam_no == 2:     #  片停留切削送り(等速->停止)
            t0 = 0; t1 = 0; t2 = 0; t3 = 0; t4 = z; t5 = 1; t6 = 1; t7 = 1; s0 = 0; s7 = 1; v7 = 0; n_a0 = 1;
            a_mp = 0;ratio = 0; self.a_mm=a_mm = pi/(4*(1-t4)**2/pi + 2*(1-t4)*t4); v0 = 2*(1-t4)*a_mm/pi;
            tt = [t0,t1,t2,t3,t4,t5,t6,t7]; ttl = 'katateiryu_sessakuokuri'   #片停留切削送り(等速->停止)' 
        elif cam_no == 0:     #　t0～t7を指定する
            n_a0 = 0
            ttl = 't0-t7 settei' 
                
    
        self.tt = tt; self.ttl = ttl; self.cam_no = cam_no; self.z = z;
        self.s0 = s0;  self.s7 = s7;  self.v0 = v0; 
        self.ratio=ratio; self.n_a0 = n_a0;
    
        #print(tt)
        #self.t*=t0～t7を指定する
        #print(len(tt))
        self.t0=t0 = tt[0]; self.t1=t1 = tt[1]; self.t2=t2 = tt[2]; self.t3=t3 = tt[3]
        self.t4=t4 = tt[4]; self.t5=t5 = tt[5]; self.t6=t6 = tt[6]; self.t7=t7 = tt[7]
        self.c0=c0 = 0
        self.c1=c1 = 2*(t1-t0)/pi
        self.c2=c2 = t2-t1
        self.c3=c3 = 2*(t3-t2)/pi
        self.c4=c4 = t4-t3
        self.c5=c5 = 2*(t5-t4)/pi
        self.c6=c6 = t6-t5
        self.c7=c7 = 2*(t7-t6)/pi
  
        c_tmp1 = s7-s0-self.v0*(t7-t0)
        c_tmp2 = c7**2+0.5*(c6**2)-(c5**2)+c6*(t7-t6)+c5*(t7-t4)
        c_tmp3 = c3**2+0.5*(c2**2)-(c1**2)+c3*(t7-t3)+c2*(t7-t2)+c1*(t7-t0)
        tmp = -ratio*c_tmp2+c_tmp3
        #print(tmp,c_tmp1,c_tmp2,c_tmp3)
        if tmp != 0:
            self.a_mp=a_mp = c_tmp1/(-ratio*c_tmp2+c_tmp3)
        if  cam_no != 2:
            self.a_mm=a_mm = ratio*a_mp
     
        self.v1=v1 = c1*a_mp+self.v0
        self.v2=v2 = c2*a_mp + v1
        self.v3=v3 = c3*a_mp + v2
        self.v4=v4 = v3
        self.v5=v5 = -c5*a_mm + v4
        self.v6=v6 = -a_mm*c6 + v5
        self.v7=v7 = -c7*a_mm + v6

        self.s1=s1 = -(c1**2)*a_mp + v1*(t1-t0) + s0
        self.s2=s2 = a_mp/2*c2*c2 + v1*c2 + s1
        self.s3=s3 = c3*c3*a_mp + v2*(t3-t2) + s2
        self.s4=s4 = v3*c4 + s3
        self.s5=s5 = c5*c5*a_mm+v5*(t5-t4) + s4
        self.s6=s6 = -a_mm/2*c6*c6 + v5*c6 + s5
        self.s7=s7 = -c7*c7*a_mm + v6*(t7-t6) + s6
    
    #---------------------------------------------------------------
    #    t=0-1 の値を与えると s:Stroke v:速度 a:加速度 を返す   
    #---------------------------------------------------------------
    def sva(self,t):
        #print(t,self.t1,self.a_mp)
        if(self.t0 <= t < self.t1):
            p = (t-self.t0)/self.c1
            #p = p_rad*180/piuniversal_cam.py
            a = self.a_mp*np.sin(p)
            j = self.a_mp/self.c1*np.cos(p)
            v = self.c1*self.a_mp*(1-np.cos(p))+self.v0
            s = -(self.c1**2)*self.a_mp*np.sin(p) + self.v1*(t-self.t0) + self.s0
        elif(self.t1 <= t < self.t2):
            p = t-self.t1
            a = self.a_mp; j = 0
            v = self.a_mp*p + self.v1
            s = self.a_mp/2*p*p + self.v1*p + self.s1
        elif(self.t2 <= t < self.t3):
            p = (t-self.t2)/self.c3
            #p = p_rad*180/pi
            a = self.a_mp*np.cos(p)   
            j = -self.a_mp/self.c3*np.sin(p)
            v = self.c3*self.a_mp*np.sin(p) + self.v2
            v_m = self.v3
            s = -(self.c3**2)*self.a_mp*(np.cos(p)-1) + self.v2*(t-self.t2) + self.s2
        elif(self.t3 <= t < self.t4):
            a = 0; j = 0; v = self.v3
            s = self.v3*(t-self.t3) + self.s3
        elif(self.t4 <= t < self.t5):
            p = (t-self.t4)/self.c5
            #p = p_rad*180/pi
            a = -self.a_mm*np.sin(p)
            j = -self.a_mm/self.c5*np.cos(p)
            v = self.c5*self.a_mm*(np.cos(p)-1) + self.v4
            s = self.c5**2*self.a_mm*np.sin(p) + self.v5*(t-self.t4) + self.s4
        elif(self.t5 <= t < self.t6):
            p = t-self.t5
            a = -self.a_mm; j=0
            v = -self.a_mm*p + self.v5
            s = -(self.a_mm/2.0*(p**2)) + self.v5*p + self.s5
        #elif(self.t6 <= t < self.t7):
        elif(self.t6 <= t <= self.t7):
            if(self.c7 == 0):
                p=0; a = -self.a_mm; j = 0
            else:
                p = (t-self.t6)/self.c7
                #p = p_rad*180/pi
                a = -self.a_mm*np.cos(p)
                j = self.a_mm/self.c7*np.sin(p)
            v = -self.c7*self.a_mm*np.sin(p) + self.v6
            s = self.c7**2*self.a_mm*(np.cos(p)-1) + self.v6*(t-self.t6) + self.s6
        """
        elif(self.t7 <= t):
            if(self.n_a0 == 2):      # 両停留
                a = 0; j = 0; v = self.v7
                #s = s7
                s = self.s7+self.v7*(t-self.t7)
            elif(self.n_a0 == 1): # 片停留 
                a = -self.a_mm; j = 0; s = 1; v = self.v7
            elif(self.n_a0 == 0):    # 無停留 */
                a = -self.a_mm; j = 0; v = self.v7
                s = self.s7+self.v7*(t-self.t7)
        """   
            #print('t=',t,'  s=',s,'  a=',a)
        
            #print('S= %f  V= %f  A= %f' % (s,v,a))
        return (s, v, a)
        #return (round(s,10), round(v,10), round(a,10))      # 少数点以下10桁で丸める

    def sva_graph(self, sva_all, max_x):    # sva全リスト、x軸最大値
        max_s = max(sva_all[0]); min_s = min(sva_all[0]) 
        max_v = max(sva_all[1]); min_v = min(sva_all[1])
        max_a = max(sva_all[2]); min_a = min(sva_all[2])
        
        dx = max_x/len(sva_all[0])
        start = 0; end = max_x
        x = np.arange(start,end,dx)  # 0から1まで0.1刻みで生成
        
        if max_x == 1:      #無次元と判断（たまたまt_h = 1の場合も有り得るが)
            plt.plot(x,sva_all[0],label="Stroke   %3.2f : %3.2f" % (max_s, min_s),color='g')
            plt.plot(x,sva_all[1],label="Velocity %3.2f : %3.2f" % (max_v, min_v),color='b')
            plt.plot(x,sva_all[2],label="Accel    %3.2f : %3.2f" % (max_a, min_a),color='r')
            plt.title('%s(NoneDim)' % self.ttl)
        else:    
            plt.plot(x,sva_all[0],label="Stroke   %3.2f : %3.2f[rad]" % (max_s, min_s),color='g')
            plt.plot(x,sva_all[1],label="Velocity %3.2f : %3.2f[rad/sec]" % (max_v, min_v),color='b')
            plt.plot(x,sva_all[2],label="Accel    %3.2f : %3.2f[rad/sec2]" % (max_a, min_a),color='r')
            plt.title(self.ttl)

        plt.legend()
        plt.show()


    #---------------------------------------------------------------------------------------------
    #   s(変位) v(速度) a(加速度) の全ての配列を返す(無次元)。nn:配列サイズ(Ex.100)  mode = 1でグラフ作図 
    #---------------------------------------------------------------------------------------------
    def sva_nn(self,nn,mode): # mode == 1 グラフ作図(無次元)
        sva_all = [[],[],[]]   # s,v,a の配列(リスト) 
        tmp=[]
        cam = Universal_cam(self.cam_no,self.z, self.ratio, *self.tt)
        for i in range(nn+1):  # i = 0 - 100
            t = i/nn
            tmp = cam.sva(t)
            #print(tmp)
            sva_all[0].append(tmp[0])
            sva_all[1].append(tmp[1])
            sva_all[2].append(tmp[2])
        
        if mode == 1:
            max_x = 1
            cam.sva_graph(sva_all,max_x)
            #print(self.sva_all[0])

        return  sva_all

    #---------------------------------------------------------------------------------------------------
    #   s(変位) v(速度) a(加速度) の全ての配列を返す(有次元)。nn:配列サイズ(Ex.100)  mode = 1でグラフ作図 
    #---------------------------------------------------------------------------------------------------
    def sva_nn_real(self, nn, mode, h, t_h):
        sva_all_real = [[],[],[]]
        #sva_all = self.sva_nn(nn, 0)
        cam = Universal_cam(self.cam_no,self.z, self.ratio, *self.tt)

        for i in range(nn+1):  # i = 0 - 100
            t = i/nn
            tmp = cam.sva(t)
            sva_all_real[0].append(tmp[0]*h)
            sva_all_real[1].append(tmp[1]*h/t_h)
            sva_all_real[2].append(tmp[2]*h/t_h**2)
        
        if mode == 1:
            cam.sva_graph(sva_all_real,t_h)
            
        return sva_all_real
 

    #-----------------------------------------------------
    #   s(変位) からt(無次元時間)を逆算する
    #-----------------------------------------------------

    def s2t(self, target_s, gosa0=0.0000001):
        if target_s > 1.0:
            print('Target_sが不正です')
            return None

        cam = Universal_cam(self.cam_no, self.z, self.ratio, *self.tt)
        #gosa0 = 0.0001
        t0 = 0.0 ; t1 = 1.0; dt = 0.1   #初期値
        def zenkin(t0, t1, dt):
            nn = round((t1-t0)/dt)+1    # int ではなく　四捨五入のround
            lst = [t0+i*dt for i in range(nn)] # t=t0; t<=t1; t+=dtと同じ
            #print(lst)
            for t in lst:
                (s, v, a) = cam.sva(t)
                gosa = abs(target_s - s)
                #print(dt, t, s, (target_s - s))
                if t == t0:
                    pre_pm = (target_s - s)/abs(target_s - s)
                    #print('pre_pm= %d' % pre_pm)
                    continue
                if gosa < gosa0:
                    break
                pm = (target_s - s)/gosa

                if pm != pre_pm:    # 符号が反転
                    #print('反転しました')
                    break   # for loop end
                         
            return t, gosa

        while True:
            (tmp_t, tmp_gosa) = zenkin(t0, t1, dt)
            #print('誤差= %1.8f' % tmp_gosa)
            if tmp_gosa < gosa0:    # 目標達成!
                break
            else:       # まだ誤差が大きい
                t0 = tmp_t - dt
                t1 = tmp_t
                dt = dt/10
                
        return tmp_t



if __name__ == "__main__":

    cam26 = Universal_cam(cam_no = 26)  # 標準変形正弦
    #all_sva26 = cam26.sva_nn(100,1)           # 100分割 plt on
    #print(cam26.sva26(0.5))
    #print(all_sva26[0])          # 変位の全てのリスト   
    #print(all_sva26[0][1])       # 変位の2番目の値   
    #print(all_sva26[1])          # 速度の全てのリスト   
    #cam1 =  Universal_cam(cam_no = 1, z = 0.8)   # 早送り
    #all_sva1 = cam1.sva_nn(100,1)  
    #cam2 =  Universal_cam(cam_no = 2)   # 切削送り 
    #all_sva2 = cam2.sva_nn(100,1)  
    #print(all_sva26[0])     # Stroke

    # s -> t 逆算
    target_s = 0.2; gosa = 0.00001
    target_t = cam26.s2t(target_s, gosa)

    print('target_s= %1.2f  starget_t= %1.8f' % (target_s,target_t))

